@@ -171,8 +171,8 @@ namespace atlas
   cblas_syrk
     (
     const enum CBLAS_ORDER Order, const enum CBLAS_UPLO Uplo, const enum CBLAS_TRANSPOSE Trans,
-    const int N, const int K, const double alpha,
-    const double* A, const int lda, const double beta, double* C, const int ldc
+    const int N, const int K, const eT alpha,
+    const eT* A, const int lda, const eT beta, eT* C, const int ldc
     )
     {
     arma_type_check((is_supported_blas_type<eT>::value == false));
